using System;
using System.IO;
using System.Runtime.InteropServices;
namespace PHS
{
    public partial class Methods
    {
        static string _statement; // This is the statement that appears in the MSIcode. Example: "into variable MyMacAddresses".
        static string _variables; // This is the comma-delimited text of variable names and their data. It's what we pass back on the second call.
        [ComVisible(true), Guid("05010770-B346-4993-B2FC-DD5DC89AC9D2")]
        public delegate bool RunTimeCallback(Int32 percentage, string text);
        
        class Selection // This class is populated with relevant data from the SelectionForm object.
        {
            public string VariableName; // This contains the variable name entered on the form.
            public bool Cancelled; // This is set true only if the Cancel button is pressed.
        }
        
        public int RunTimeExecute(Int32 window, string variables, string statement, Int32 callback, ref Int32 returnValue, ref string updatedVariables, bool firstIteration)
        {
#if ENABLEJITDEBUGGING
            System.Diagnostics.Debugger.Launch();
#endif
#if SHOWPARAMETERS
            DisplayRuntimeExecuteParameters(window, variables, statement, callback, returnValue, updatedVariables, firstIteration);
#endif
            // The first time we're called is where we do the real work. Deconstructing the statement parameter tells us what variable we return the value into.

            if(firstIteration)
            {
                // Deconstruct the statement to get its associated variable name.

                string varName;
                DeconstructState(statement, out varName);
                varName = varName.ToUpper().Trim();

                // The uber-list of variables passed to us by InstallAware is a comma delimited string. We split it into an array.

                string[] items = variables.Split(',');

                // We need to search the array and see if we've been given a variable name. That is, did the user create a variable using
                // the Set Variable statement in the MSIcode? If not, there's little point doing anything if we can't return a value.

                bool found = false;
                int dataIndex = 0;
                for(int i = 0; i < items.Length; i += 2) // Variable names are in index positions 0, 2, 4... The data for them is in 1, 3, 5...
                {
                    string iaVariable = items[i].ToUpper().Trim();
                    if(iaVariable != varName)
                    {
                        continue;
                    }

                    // We found the variable name used in the statement.

                    found = true;
                    dataIndex = i + 1; // The data value for the variable is in the next element.
                    break;
                }
                if(!found) // It's pointless doing anything so we'll give back what we were given in the first place.
                {
                    _variables = variables;
                    return _variables.Length;
                }

                // Here, the code below replaces the currently defined plugin variable with appropriate data. It's recommended that all values
                // returned are double-quoted. This is especially important if any value might contain embedded spaces. If we detect any exception
                // getting the MAC addresses, return an error indication to the caller.

                items[dataIndex] = string.Format("\"{0}\"", MacAddresses);

                // When we've finished doing what we need recombine all the variables into a comma-delimited string and let InstallAware know
                // how much data (length) is available to be sent back. The second call will return the data.

                _variables = string.Join(",", items);
                return _variables.Length;
            }

            // On the second call, we now give back the data. The first call told InstallAware how much data we have, not the data itself. We
            // always give back all of the variables that InstallAware gave us. We update the ones we need but we always give back all others.

            updatedVariables = _variables;
            return 0;
        }

        public int DesignTimeEdit(uint window, string statement, ref string newStatement, bool displayDialog)
        {
#if ENABLEJITDEBUGGING
            System.Diagnostics.Debugger.Launch();
#endif
#if SHOWPARAMETERS
            DisplayDesignTimeEditParameters(window, statement, newStatement, displayDialog);
#endif
            // On the first call the display dialog flag is true so display the selection form and get the user's response.

            if(displayDialog)
            {
                // The statement parameter has two values we need to handle:
                // 1. If it's null it tells us the plugin is being dragged from the toolbox onto the MSIcode window. In other words, a new
                //    reference to the plugin is being added.
                // 2. If it contains text the user is editing an existing MSIcode plugin statement in the MSIcode window. In this case
                //    we need to extract the variable name so we can display it in the dialog for editing.

                string variableName = string.Empty;
                if(!string.IsNullOrEmpty(statement))
                {
                    DeconstructState(statement, out variableName);
                }

                // Display the dialog and wait for the user to dismiss it.

                Selection selection = DisplaySelectionForm(variableName);
                if(selection.Cancelled)
                {
                    return -1; // This tells InstallAware we don't want to do anything. No MSIcode will be created.
                }

                // Construct the statement to be emitted to the MSIcode. The only thing InstallAware needs at this time is the
                // length of the statement text. On the second call we give it the actual text.

                variableName = selection.VariableName;
                _statement = ConstructStatement(variableName); // Save the statement text for the second call.
                return _statement.Length;
            }

            // On the second call give InstallAware the statement text to be displayed in the MSIcode window.

            newStatement = _statement;
            return 0;
        }

        public int DesignTimeText(string statement, ref string text)
        {
#if ENABLEJITDEBUGGING
            System.Diagnostics.Debugger.Launch();
#endif
#if SHOWPARAMETERS
            DisplayDesignTimeTextParameters(statement, text);
#endif
            // The text parameter is a funny creature for two reasons:
            // 1. If it's null we return the length of the statement we've been given.
            // 2. If it contains something, usually a single space, then we return the statement text.

            if(string.IsNullOrEmpty(text))
            {
                return statement.Length;
            }
            text = statement;
            return 0;
        }

// ReSharper disable RedundantAssignment
        public bool CompileTimeBuild(string moduleFolder, Int32 msiHandle, string state, string buildPath, bool build, ref string fetch)
// ReSharper restore RedundantAssignment
        {
#if ENABLEJITDEBUGGING
            System.Diagnostics.Debugger.Launch();
#endif
#if SHOWPARAMETERS
            DisplayCompileTimeBuildParameters(moduleFolder, msiHandle, state, buildPath, build, fetch);
#endif
            // If we're building the setup media then copy the plugin and any additional assemblies it needs, as well as the bridge assemblies and ini files.

            fetch = "True";
            if(!build)
            {
                return true;
            }
            string[] files =
            {
                "MacAddressesIde.dll",
                "MacAddressesRuntime.dll",
                "MacAddressesAccessory.dll",
                "DesignTime_NetBridge.dll",
                "DesignTime_NetBridge.ini",
                "RunTime_NetBridge.dll",
                "RunTime_NetBridge.ini"
            };
            foreach(string file in files)
            {
                string srcPath = moduleFolder + file;
                string destPath = buildPath + file;
                File.Copy(srcPath, destPath, true);
            }
            return true;
        }
    }
}
